﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Charts;
using System.Windows.Data;

namespace XamDataChartIntersection
{
    public partial class MainPage : UserControl
    {
        public List<Data> data;
        public MainPage()
        {
            InitializeComponent();
            data = new List<Data>();
            data.Add(new Data() { XValue = DateTime.Now.AddHours(1), YValue = 12 });
            data.Add(new Data() { XValue = DateTime.Now.AddHours(2), YValue = 11 });
            data.Add(new Data() { XValue = DateTime.Now.AddHours(5), YValue = 11 });
            data.Add(new Data() { XValue = DateTime.Now.AddHours(-1), YValue = 5 });
            data.Add(new Data() { XValue = DateTime.Now.AddHours(9), YValue = 8 });
            data.Add(new Data() { XValue = DateTime.Now.AddHours(-13), YValue = 12 });

            this.DataContext = data;

            List<Point> intersect = new List<Point>();
            for (int i = 0; i < data.Count; i++)
            {
                if (i != data.Count - 1 && data[i].YValue > 10 && data[i + 1].YValue < 10)
                {
                    Point p1 = new Point(i, data[i].YValue);
                    Point p2 = new Point(i + 1, data[i + 1].YValue);
                    Point q1 = new Point(i, 10);
                    Point q2 = new Point(i + 1, 10);
                    Point p = findIntersection(p1, p2, q1, q2);
                    intersect.Add(p);
                }
                if (i != data.Count - 1 && data[i].YValue < 10 && data[i + 1].YValue > 10)
                {
                    Point p1 = new Point(i, data[i].YValue);
                    Point p2 = new Point(i + 1, data[i + 1].YValue);
                    Point q1 = new Point(i, 10);
                    Point q2 = new Point(i + 1, 10);
                    Point p = findIntersection(p1, p2, q1, q2);
                    intersect.Add(p);
                }
            }

            xamDataChart1.Series[2].ItemsSource = intersect;

        }

        class Point
        {
            public double x { get; set; }
            public double y { get; set; }
            public Point(double x, double y)
            {
                this.x = x;
                this.y = y;
            }

            void set(double x, double y)
            {
                this.x = x;
                this.y = y;
            }
        }


        Point findIntersection(Point p1, Point p2, Point p3, Point p4)
        {
            double xD1, yD1, xD2, yD2, xD3, yD3;
            double dot, deg, len1, len2;
            double segmentLen1, segmentLen2;
            double ua, ub, div;

            // calculate differences  
            xD1 = p2.x - p1.x;
            xD2 = p4.x - p3.x;
            yD1 = p2.y - p1.y;
            yD2 = p4.y - p3.y;
            xD3 = p1.x - p3.x;
            yD3 = p1.y - p3.y;

            // calculate the lengths of the two lines  
            len1 = Math.Sqrt((double)(xD1 * xD1 + yD1 * yD1));
            len2 = Math.Sqrt((double)(xD2 * xD2 + yD2 * yD2));

            // calculate angle between the two lines.  
            dot = (xD1 * xD2 + yD1 * yD2); // dot product  
            deg = dot / (len1 * len2);

            // if abs(angle)==1 then the lines are parallell,  
            // so no intersection is possible  
            if (Math.Abs(deg) == 1) return null;

            // find intersection Pt between two lines  
            Point pt = new Point(0, 0);
            div = yD2 * xD1 - xD2 * yD1;
            ua = (xD2 * yD3 - yD2 * xD3) / div;
            ub = (xD1 * yD3 - yD1 * xD3) / div;
            pt.x = p1.x + ua * xD1;
            pt.y = p1.y + ua * yD1;

            // calculate the combined length of the two segments  
            // between Pt-p1 and Pt-p2  
            xD1 = pt.x - p1.x;
            xD2 = pt.x - p2.x;
            yD1 = pt.y - p1.y;
            yD2 = pt.y - p2.y;
            segmentLen1 = Math.Sqrt((double)xD1 * xD1 + yD1 * yD1) + Math.Sqrt((double)xD2 * xD2 + yD2 * yD2);

            // calculate the combined length of the two segments  
            // between Pt-p3 and Pt-p4  
            xD1 = pt.x - p3.x;
            xD2 = pt.x - p4.x;
            yD1 = pt.y - p3.y;
            yD2 = pt.y - p4.y;
            segmentLen2 = Math.Sqrt((double)xD1 * xD1 + yD1 * yD1) + Math.Sqrt((double)(xD2 * xD2 + yD2 * yD2));

            // if the lengths of both sets of segments are the same as  
            // the lenghts of the two lines the point is actually  
            // on the line segment.  

            // if the point isn’t on the line, return null  
            if (Math.Abs(len1 - segmentLen1) > 0.01 || Math.Abs(len2 - segmentLen2) > 0.01)
                return null;

            // return the valid intersection  
            return pt;
        }


    }

    public class Data
    {
        public DateTime XValue { get; set; }
        public int YValue { get; set; }
    }

    public class CountConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            if (value != null)
            {
                return (int)value - 1;
            }
            return 0;
        }

        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
